#!/usr/bin/env python

# Written by Bram Cohen
# modified for multitracker by John Hoffman
# see LICENSE.txt for license information

#import PSYCO
#if PSYCO.psyco:
#    try:
#        import psyco
#        assert psyco.__version__ >= 0x010100f0
#        psyco.full()
#    except:
#        pass

from sys import argv, version

from btcompletedir import completedir
from btmakemetafile import make_meta_file
from threading import Event, Thread
from BitTorrent.bencode import bdecode
from os.path import join, isdir
from os import getcwd, path
from wxPython.wx import *
true = 1
false = 0

wxEVT_INVOKE = wxNewEventType()

def EVT_INVOKE(win, func):
    win.Connect(-1, -1, wxEVT_INVOKE, func)

class InvokeEvent(wxPyEvent):
    def __init__(self, func, args, kwargs):
        wxPyEvent.__init__(self)
        self.SetEventType(wxEVT_INVOKE)
        self.func = func
        self.args = args
        self.kwargs = kwargs

class AnnounceFile:
    def __init__(self, announceconf):
        self.announcelst = []
        self.piece_size = 0
        self.announcedefault = ""
        self.announceconf = announceconf

    def readConfig(self):        
        f = open(self.announceconf, 'rb')
        while True:
            configline = f.readline()        
            if configline == "" or configline == "\n":
                break
            else:
                configmap = configline.split("=")
                if configmap[0] == "piece_size":
                    self.piece_size = int(configmap[1][0:-1])
                elif configmap[0] == "announce":
                    self.announcelst.append(configmap[1][0:-1])
                elif configmap[0] == "announcedefault":
                    self.announcelst.append(configmap[1][0:-1])
                    self.announcedefault = configmap[1][0:-1]

        f.close()
        return self.announcedefault, self.announcelst, self.piece_size

    def saveDefault(self, piece, url):
        numslot = -1
        self.piece_size = piece
        print "piece_size = %d" % piece
        for i in range(0, len(self.announcelst)):
           if self.announcelst[i] == url:
               numslot = i
               break
        if numslot == -1:   #current URL is bad
            if len(self.announcelst) != 0:
                self.announcedefault = self.announcelst[0]
                
        f = open(self.announceconf, 'wb')
        f.writelines("piece_size=" + str(piece) + "\n")        
        for i in range(0, len(self.announcelst)):
            if self.announcelst[i] == self.announcedefault:
                f.writelines("announcedefault="+self.announcedefault+"\n")
            else:
                f.writelines("announce="+self.announcelst[i]+"\n")
        f.close()

        
        
    def addAnnounce(self, url):
        self.announcelst.append(url)
        self.announcedefault = url
        f = open(self.announceconf, 'wb')
        f.writelines("piece_size=" + str(self.piece_size) + "\n")        
        for i in range(0, len(self.announcelst)):
            if self.announcelst[i] == self.announcedefault:
                f.writelines("announcedefault="+self.announcedefault+"\n")
            else:
                f.writelines("announce="+self.announcelst[i]+"\n")
        f.close()

    def removeAnnounce(self, url):
        numslot = -1
        for i in range(0, len(self.announcelst)):
           if self.announcelst[i] == url:
               numslot = i
               break
        if numslot == -1:
            return
        else:
            del self.announcelst[i]
            if self.announcedefault == url:
                if len(self.announcelst) != 0:
                    self.announcedefault = self.announcelst[0]
                else:
                    self.announcedefault = ''
        
        f = open(self.announceconf, 'wb')
        f.writelines("piece_size=" + str(self.piece_size) + "\n")        
        for i in range(0, len(self.announcelst)):
            if self.announcelst[i] == self.announcedefault:
                f.writelines("announcedefault="+self.announcedefault+"\n")
            else:
                f.writelines("announce="+self.announcelst[i]+"\n")
        f.close()

class DownloadInfo:
    def __init__(self, parent, abcpath):

        self.parent = parent
        frame = wxFrame(self.parent, -1, 'BitTorrent Torrent File Maker', size = wxSize(550, 410))
        self.frame = frame

        if (sys.platform == 'win32'):
            self.icon = wxIcon(path.join(abcpath, 'icon_abc.ico'), wxBITMAP_TYPE_ICO)
            self.frame.SetIcon(self.icon)

        self.announceconf = AnnounceFile(path.join(abcpath, "announce.lst"))        
        # Read config file
        #########################
        self.announcedefault, self.announcelst, self.piece_size = self.announceconf.readConfig()

        panel = wxPanel(frame, -1)

        gridSizer = wxFlexGridSizer(cols = 2, rows = 2, vgap = 0, hgap = 8)
        
        gridSizer.Add(wxStaticText(panel, -1, 'make torrent of:'))

        b = wxBoxSizer(wxHORIZONTAL)
        self.dirCtl = wxTextCtrl(panel, -1, '')
        b.Add(self.dirCtl, 1, wxEXPAND)
        b.Add(10, 10, 0, wxEXPAND)
        
        button = wxButton(panel, -1, 'dir', size = (30,20))
        EVT_BUTTON(frame, button.GetId(), self.selectdir)
        b.Add(button, 0)

        button2 = wxButton(panel, -1, 'file', size = (30,20))
        EVT_BUTTON(frame, button2.GetId(), self.selectfile)
        b.Add(button2, 0)

        gridSizer.Add(b, 0, wxEXPAND)
        gridSizer.Add(wxStaticText(panel, -1, ''))
        gridSizer.Add(wxStaticText(panel, -1, ''))

        gridSizer.Add(wxStaticText(panel, -1, 'announce url:'))

        b = wxBoxSizer(wxHORIZONTAL)
        
        #self.annCtl = wxTextCtrl(panel, -1, '')
        self.annCtl = wxComboBox(panel, -1, self.announcedefault,choices=self.announcelst, style=wxCB_DROPDOWN)
        b.Add(self.annCtl, 1, wxEXPAND)
        b.Add(10, 10, 0, wxEXPAND)
        
        button = wxButton(panel, -1, 'Add', size = (30,20))
        EVT_BUTTON(frame, button.GetId(), self.addAnnounce)
        b.Add(button, 0)

        button2 = wxButton(panel, -1, 'Remove', size = (60,20))
        EVT_BUTTON(frame, button2.GetId(), self.removeAnnounce)
        b.Add(button2, 0)        

        # Change here to combobox


        gridSizer.Add(b, 0, wxEXPAND)
        gridSizer.Add(wxStaticText(panel, -1, ''))
        gridSizer.Add(wxStaticText(panel, -1, ''))

        a = wxFlexGridSizer(cols = 1)
        a.Add(wxStaticText(panel, -1, 'announce list:'))
        a.Add(wxStaticText(panel, -1, ''))
        abutton = wxButton(panel, -1, 'copy\nannounces\nfrom\ntorrent', size = (50,70))
        EVT_BUTTON(frame, abutton.GetId(), self.announcecopy)
        a.Add(abutton, 0, wxEXPAND)
        gridSizer.Add(a, 0, wxEXPAND)
        
        self.annListCtl = wxTextCtrl(panel, -1, '\n\n\n\n\n', wxPoint(-1,-1), (400,120),
                                            wxTE_MULTILINE|wxHSCROLL|wxTE_DONTWRAP)
        gridSizer.Add(self.annListCtl, -1, wxEXPAND)

        gridSizer.Add(wxStaticText(panel, -1, ''))
        exptext = wxStaticText(panel, -1,
                "a list of announces separated by commas " +
                "or whitespace and on several lines -\n" +
                "trackers on the same line will be tried randomly," +
                "and all the trackers on one line\n" +
                "will be tried before the trackers on the next line.")
        exptext.SetFont(wxFont(6, wxDEFAULT, wxNORMAL, wxNORMAL, false))
        gridSizer.Add(exptext)

        gridSizer.Add(wxStaticText(panel, -1, ''))
        gridSizer.Add(wxStaticText(panel, -1, ''))

        gridSizer.Add(wxStaticText(panel, -1, 'piece size:'))
        self.piece_length = wxChoice(panel, -1,
                 choices = ['automatic', '2MiB', '1MiB', '512KiB', '256KiB', '128KiB', '64KiB', '32KiB'])
        self.piece_length_list = [0,       21,     20,      19,       18,       17,      16,      15]
        self.piece_length.SetSelection(self.piece_size)
        gridSizer.Add(self.piece_length)
        
        gridSizer.Add(wxStaticText(panel, -1, ''))
        gridSizer.Add(wxStaticText(panel, -1, ''))

        gridSizer.Add(wxStaticText(panel, -1, 'comment:'))
        self.commentCtl = wxTextCtrl(panel, -1, '')
        gridSizer.Add(self.commentCtl, 0, wxEXPAND)

        gridSizer.AddGrowableCol(1)
 
        border = wxBoxSizer(wxVERTICAL)
        border.Add(gridSizer, 0, wxEXPAND | wxNORTH | wxEAST | wxWEST, 25)

        btnbox = wxBoxSizer(wxHORIZONTAL)                
        b3 = wxButton(panel, -1, 'Save as default config')
        btnbox.Add(b3, 1, wxEXPAND)
        btnbox.Add(10, 10, 0, wxEXPAND)

        b2 = wxButton(panel, -1, 'Make Torrent')
        btnbox.Add(b2, 1, wxEXPAND)
        btnbox.Add(10, 10, 0, wxEXPAND)

        b4 = wxButton(panel, -1, 'Close')
        btnbox.Add(b4, 1, wxEXPAND)
        btnbox.Add(10, 10, 0, wxEXPAND)
        
        border.Add(10, 10, 1, wxEXPAND)
        border.Add(btnbox, 0, wxALIGN_CENTER | wxSOUTH, 20)
        
        EVT_BUTTON(frame, b2.GetId(), self.complete)
        EVT_BUTTON(frame, b3.GetId(), self.saveconfig)
        EVT_BUTTON(frame, b4.GetId(), self.closewin)
        panel.SetSizer(border)
        panel.SetAutoLayout(true)

#        panel.DragAcceptFiles(true)
#        EVT_DROP_FILES(panel, self.selectdrop)

    def closewin(self, x):
        self.frame.Destroy()
        
    def saveconfig(self, x):
        self.piece_length.GetSelection()
        self.announceconf.saveDefault(self.piece_length.GetSelection(), self.annCtl.GetValue())
        
    def addAnnounce(self, x):
        announceurl = self.annCtl.GetValue()
        if announceurl == "":
            return
        self.annCtl.Append(announceurl)
        self.announceconf.addAnnounce(announceurl)
        
    def removeAnnounce(self, x):
        if self.annCtl.GetSelection() == -1:
            dlg = wxMessageDialog(self.frame, "Please select announce url from combo box", "Error", wxICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        else:
            announceurl = self.annCtl.GetValue()
            i = self.annCtl.GetSelection()
            self.announceconf.removeAnnounce(announceurl)
            self.annCtl.Delete(i)
        
    def selectdir(self, x):
        dl = wxDirDialog(self.frame, style = wxDD_DEFAULT_STYLE | wxDD_NEW_DIR_BUTTON)
        if dl.ShowModal() == wxID_OK:
            self.dirCtl.SetValue(dl.GetPath())

    def selectfile(self, x):
        dl = wxFileDialog (self.frame, 'Choose file or directory to use', '', '', '', wxOPEN)
        if dl.ShowModal() == wxID_OK:
            self.dirCtl.SetValue(dl.GetPath())

    def selectdrop(self, x):
        print x

        list = x.m_files
        self.dirCtl.SetValue(x[0])

    def announcecopy(self, x):
        dl = wxFileDialog (self.frame, 'Choose .torrent file to use', '', '', '*.torrent', wxOPEN)
        if dl.ShowModal() == wxID_OK:
            try:
                h = open(dl.GetPath(), 'rb')
                metainfo = bdecode(h.read())
                h.close()
                self.annCtl.SetValue(metainfo['announce'])
                if metainfo.has_key('announce-list'):
                    list = []
                    for tier in metainfo['announce-list']:
                        for tracker in tier:
                            list += [tracker, ', ']
                        del list[-1]
                        list += ['\n']
                    liststring = ''
                    for i in list:
                        liststring += i
                    self.annListCtl.SetValue(liststring+'\n\n')
                else:
                    self.annListCtl.SetValue('')
            except:
                return

    def getannouncelist(self):
        list = []
        for t in self.annListCtl.GetValue().split('\n'):
            tier = []
            t = t.replace(',',' ')
            for tr in t.split(' '):
                if tr != '':
                    tier += [tr]
            if len(tier)>0:
                list.append(tier)
        return list
    
    def complete(self, x):
        if self.dirCtl.GetValue() == '':
            dlg = wxMessageDialog(self.frame, message = 'You must select a\n file or directory', 
                caption = 'Error', style = wxOK | wxICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return
        params = {'piece_size_pow2': self.piece_length_list[self.piece_length.GetSelection()]}
        annlist = self.getannouncelist()
        if len(annlist)>0:
            params['real_announce_list'] = annlist
        comment = self.commentCtl.GetValue()
        if comment != '':
            params['comment'] = comment
        try:
            CompleteDir(self.dirCtl.GetValue(), self.annCtl.GetValue(), params)
        except:
            print_exc()


from traceback import print_exc

class CompleteDir:
    def __init__(self, d, a, params):
        self.d = d
        self.a = a
        self.params = params
        self.flag = Event()
        self.separatetorrents = false

        if isdir(d):
            self.choicemade = Event()
            frame = wxFrame(None, -1, 'BitTorrent make torrent', size = (1,1))
            self.frame = frame
            panel = wxPanel(frame, -1)
            gridSizer = wxFlexGridSizer(cols = 1, vgap = 8, hgap = 8)
            gridSizer.AddGrowableRow(1)
            gridSizer.Add(wxStaticText(panel, -1,
                    'Do you want to make a separate .torrent'),0,wxALIGN_CENTER)
            gridSizer.Add(wxStaticText(panel, -1,
                    'for every item in this directory?'),0,wxALIGN_CENTER)
            gridSizer.Add(wxStaticText(panel, -1, ''))

            b = wxFlexGridSizer(cols = 3, hgap = 10)
            yesbut = wxButton(panel, -1, 'Yes')
            def saidyes(e, self = self):
                self.frame.Destroy()
                self.separatetorrents = true
                self.begin()
            EVT_BUTTON(frame, yesbut.GetId(), saidyes)
            b.Add(yesbut, 0)

            nobut = wxButton(panel, -1, 'No')
            def saidno(e, self = self):
                self.frame.Destroy()
                self.begin()
            EVT_BUTTON(frame, nobut.GetId(), saidno)
            b.Add(nobut, 0)

            cancelbut = wxButton(panel, -1, 'Cancel')
            def canceled(e, self = self):
                self.frame.Destroy()                
            EVT_BUTTON(frame, cancelbut.GetId(), canceled)
            b.Add(cancelbut, 0)
            gridSizer.Add(b, 0, wxALIGN_CENTER)
            border = wxBoxSizer(wxHORIZONTAL)
            border.Add(gridSizer, 1, wxEXPAND | wxALL, 4)
            
            panel.SetSizer(border)
            panel.SetAutoLayout(true)
            frame.Show()
            border.Fit(panel)
            frame.Fit()
            
        else:
            self.begin()

    def begin(self):
        if self.separatetorrents:
            frame = wxFrame(None, -1, 'BitTorrent make directory', size = wxSize(550, 250))
        else:
            frame = wxFrame(None, -1, 'BitTorrent make torrent', size = wxSize(550, 250))
        self.frame = frame

        panel = wxPanel(frame, -1)
        gridSizer = wxFlexGridSizer(cols = 1, vgap = 15, hgap = 8)

        if self.separatetorrents:
            self.currentLabel = wxStaticText(panel, -1, 'checking file sizes')
        else:
            self.currentLabel = wxStaticText(panel, -1, 'building ' + self.d + '.torrent')
        gridSizer.Add(self.currentLabel, 0, wxEXPAND)
        self.gauge = wxGauge(panel, -1, range = 1000, style = wxGA_SMOOTH)
        gridSizer.Add(self.gauge, 0, wxEXPAND)
        gridSizer.Add(10, 10, 1, wxEXPAND)
        self.button = wxButton(panel, -1, 'cancel')
        gridSizer.Add(self.button, 0, wxALIGN_CENTER)
        gridSizer.AddGrowableRow(2)
        gridSizer.AddGrowableCol(0)

        g2 = wxFlexGridSizer(cols = 1, vgap = 15, hgap = 8)
        g2.Add(gridSizer, 1, wxEXPAND | wxALL, 25)
        g2.AddGrowableRow(0)
        g2.AddGrowableCol(0)
        panel.SetSizer(g2)
        panel.SetAutoLayout(true)
        EVT_BUTTON(frame, self.button.GetId(), self.done)
        EVT_CLOSE(frame, self.done)
        EVT_INVOKE(frame, self.onInvoke)
        frame.Show(true)
        Thread(target = self.complete).start()

    def complete(self):
        try:
            if self.separatetorrents:
                completedir(self.d, self.a, self.params, self.flag,
                            self.valcallback, self.filecallback)
            else:
                make_meta_file(self.d, self.a, self.params, self.flag,
                            self.valcallback, progress_percent = 1)
            if not self.flag.isSet():
                self.currentLabel.SetLabel('Done!')
                self.gauge.SetValue(1000)
                self.button.SetLabel('Close')
                self.frame.Refresh()
        except (OSError, IOError), e:
            self.currentLabel.SetLabel('Error!')
            self.button.SetLabel('Close')
            dlg = wxMessageDialog(self.frame, message = 'Error - ' + str(e), 
                caption = 'Error', style = wxOK | wxICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()

    def valcallback(self, amount):
        self.invokeLater(self.onval, [amount])

    def onval(self, amount):
        self.gauge.SetValue(int(amount * 1000))

    def filecallback(self, f):
        self.invokeLater(self.onfile, [f])

    def onfile(self, f):
        self.currentLabel.SetLabel('building ' + join(self.d, f) + '.torrent')

    def onInvoke(self, event):
        if not self.flag.isSet():
            apply(event.func, event.args, event.kwargs)

    def invokeLater(self, func, args = [], kwargs = {}):
        if not self.flag.isSet():
            wxPostEvent(self.frame, InvokeEvent(func, args, kwargs))

    def done(self, event):
        self.flag.set()
        self.frame.Destroy()

class btWxApp(wxApp):
    def OnInit(self):
        d = DownloadInfo()
        d.frame.Show(true)
        self.SetTopWindow(d.frame)
        return true

if __name__ == '__main__':
    btWxApp().MainLoop()
